#pragma once

#include <string>
#include <memory>
#include <vector>
#include <chrono>

#include "simstring\simstring.h"

class RaceDateLibrary
{
public:
	struct Race {
		enum Grade {
			kG1 = 1 << 0,
			kG2 = 1 << 1,
			kG3 = 1 << 2,
		};
		Grade			grade;				// G1EG2EG3
		std::wstring	name;				// [X
		enum GroundCondition {
			kGrass = 1 << 3,
			kDart  = 1 << 4,
		};
		GroundCondition	groundCondition;	// ŁE_[g
		enum DistanceClass {
			kSprint = 1 << 5,
			kMile	= 1 << 6,
			kMiddle	= 1 << 7,
			kLong	= 1 << 8,
		};
		DistanceClass	distanceClass;		// ZE}CEE
		std::wstring	distance;			// ̎ۂ̋
		enum Rotation {
			kRight	= 1 << 9, 
			kLeft	= 1 << 10,
			kLine	= 1 << 11,
		};
		Rotation		rotation;			// EEE
		std::wstring	location;			// ꏊ
		enum Location {
			kSapporo	= 1 << 12,
			kHakodate	= 1 << 13,
			kHukusima	= 1 << 14,
			kNiigata	= 1 << 15,
			kTokyo		= 1 << 16,
			kNakayama	= 1 << 17,
			kTyuukyou	= 1 << 18,
			kKyoto		= 1 << 19,
			kHanshin	= 1 << 20,
			kOgura		= 1 << 21,
			kOoi		= 1 << 22,
			kMaxLocationCount = 11,
		};
		Location		locationFlag;
		std::vector<std::wstring>	date;	// JÓ

		// =============================
		std::wstring	RaceName() const;
		std::wstring	GroundConditionText() const;
		std::wstring	DistanceText() const;
		std::wstring	RotationText() const;

		// ނORAʎނAND
		bool	IsMatchState(int32_t state);
	};

	// RaceDataLibrary.json ǂݍ
	bool	LoadRaceDataLibrary();

	// S^[Xg
	const std::vector<std::wstring>& GetAllTurnList() const {
		return m_allTurnList;
	}
	// ^[̑S[XXg
	const std::vector<std::vector<std::shared_ptr<Race>>>& GetTurnOrderedRaceList() const {
		return m_turnOrderedRaceList;
	}

	// 
	int		GetTurnNumberFromTurnName(const std::wstring& searchTurn);


	// ܂Ō݂̓tύX
	std::wstring	AnbigiousChangeCurrentTurn(std::vector<std::wstring> ambiguousCurrentTurn);

private:
	void	_InitDB();

	std::wstring	m_currentTurn;
	int		m_searchCount = 0;	// tsEksh~̂

	std::vector<std::wstring> m_allTurnList;

	std::unique_ptr<simstring::reader>	m_dbReader;

	std::vector<std::vector<std::shared_ptr<Race>>>	m_turnOrderedRaceList;
};

